"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var mongoose = require("mongoose");
var typedi_1 = require("typedi");
var server_core_module_1 = require("@modulfy/server-core-module");
var server_core_events_1 = require("@modulfy/server-core-events");
var __1 = require("..");
var MongoConfig = /** @class */ (function () {
    function MongoConfig() {
        this.globalContext = typedi_1.Container.get(server_core_module_1.GlobalContext);
        this.events = typedi_1.Container.get(server_core_events_1.Events);
    }
    MongoConfig.prototype.connect = function (uri) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            mongoose.set('useCreateIndex', true);
            mongoose.connect(uri, { useNewUrlParser: true, useUnifiedTopology: true });
            var db = mongoose.connection;
            mongoose.Promise = global.Promise;
            db.on('error', function (e) {
                __1.MongoDbModule.logger.error('Could not connect to MongoDB: ' + e);
                reject(e);
            });
            db.once('open', function () {
                _this.connection = db;
                _this.globalContext.setDatabase(db);
                _this.events.emit(__1.MONGODB_CONNECTED, db);
                __1.MongoDbModule.logger.info('Connected to MongoDB');
                return resolve(db);
            });
        });
    };
    MongoConfig.prototype.close = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            mongoose.connection.close(function () {
                _this.globalContext.unsetDatabase();
                _this.events.emit(__1.MONGODB_DISCONNECTED);
                resolve();
            });
        });
    };
    return MongoConfig;
}());
exports.default = new MongoConfig();
