import * as mongoose from 'mongoose';
import { IDataAccessLayer } from '@modulfy/server-dal-interface';
import { ServiceResponse } from '@modulfy/server-core-web';
import MongoDataModel from './MongoDataModel';
import { Logger } from '@modulfy/server-core-logging/dist';
export default class MongoDal<T extends mongoose.Document> implements IDataAccessLayer {
    protected logger: Logger;
    protected dataModel: MongoDataModel<T>;
    protected populate: string[];
    constructor(dataModel: MongoDataModel<T>, logger: Logger, populate?: any[]);
    insert(body: any): Promise<ServiceResponse<T>>;
    save(model: any): Promise<ServiceResponse<T>>;
    updateById(id: string, updates: any): Promise<ServiceResponse<T>>;
    delete(query: any): Promise<ServiceResponse<T>>;
    deleteById(id: string): Promise<ServiceResponse<T>>;
    findById(id: string): Promise<ServiceResponse<T>>;
    find(query: any, size: number, offset?: number, sort?: any): Promise<ServiceResponse<T[]>>;
    count(query: any): Promise<ServiceResponse<number>>;
    /**
     * Joins data of a queried row
     * @param query The query to perform
     * @param populate The populating configurations used to join other tables
     * @returns {any} The query
     */
    private populateQuery;
    /**
     * Handles the error response sent back from a mongoose query
     * @param resolve promise resolve
     * @param reject promise reject
     * @param err The error
     * @param {any} model The model
     * @returns {ServiceResponse} A service response based on if there was an error or not
     */
    handleStandardResponse(resolve: any, reject: any, err: any, model?: any): ServiceResponse<any>;
    /**
     * Helper function to standardize promise and it's return type
     *
     * @param func The function to call in the promise
     * @returns {Promise<ServiceResponse>} The promise
     */
    promise(func: (resolve: any, reject: any) => void): Promise<ServiceResponse<any>>;
    handleCaught(e: any, reject: any, res: any): any;
}
