"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var server_core_web_1 = require("@modulfy/server-core-web");
var MongoDal = /** @class */ (function () {
    function MongoDal(dataModel, logger, populate) {
        if (populate === void 0) { populate = []; }
        this.logger = logger;
        this.dataModel = dataModel;
        this.populate = populate;
    }
    MongoDal.prototype.insert = function (body) {
        var _this = this;
        return this.promise(function (resolve, reject) {
            _this.dataModel.model.create(body, function (err, model) { return _this.handleStandardResponse(resolve, reject, err, model); });
        });
    };
    ;
    MongoDal.prototype.save = function (model) {
        return this.promise(function (resolve, reject) {
            model.save(function (err) {
                if (err) {
                    return reject(new server_core_web_1.ServiceResponse(err));
                }
                return resolve(new server_core_web_1.ServiceResponse(model));
            });
        });
    };
    MongoDal.prototype.updateById = function (id, updates) {
        var _this = this;
        return this.promise(function (resolve, reject) {
            _this.findById(id).then(function (modelRes) { return modelRes.data.update(updates, function (err) { return _this.handleStandardResponse(resolve, reject, err); }).catch(reject); }).catch(reject);
        });
    };
    MongoDal.prototype.delete = function (query) {
        var _this = this;
        return this.promise(function (resolve, reject) {
            _this.dataModel.model.remove(query, function (err) { return _this.handleStandardResponse(resolve, reject, err); });
        });
    };
    MongoDal.prototype.deleteById = function (id) {
        var _this = this;
        return this.promise(function (resolve, reject) {
            return _this.dataModel.model.findOneAndRemove({ _id: id }, function (err, model) { return _this.handleStandardResponse(resolve, reject, err, model); });
        });
    };
    MongoDal.prototype.findById = function (id) {
        var _this = this;
        return this.promise(function (resolve, reject) {
            _this.populateQuery(_this.dataModel.model.find({ _id: id }).limit(1), _this.populate).exec(function (err, model) {
                if (err) {
                    return reject(new server_core_web_1.ServiceResponse(err));
                }
                if (model instanceof Array) {
                    if (model.length === 1) {
                        return resolve(new server_core_web_1.ServiceResponse(model[0]));
                    }
                    else if (model.length > 1) {
                        _this.logger.critical("Multiple rows found for ID: " + id + ". Returned Data: " + JSON.stringify(model));
                        return resolve(new server_core_web_1.ServiceResponse(model[0]));
                    }
                    return reject(new server_core_web_1.ServiceResponse("No entry found with ID: " + id, 400));
                }
                if (model) {
                    return resolve(new server_core_web_1.ServiceResponse(model));
                }
                return reject(new server_core_web_1.ServiceResponse("No entry found with ID: " + id, 400));
            });
        });
    };
    MongoDal.prototype.find = function (query, size, offset, sort) {
        var _this = this;
        if (offset === void 0) { offset = 0; }
        return this.promise(function (resolve, reject) {
            var req = _this.dataModel.model.find(query).skip(offset).limit(size);
            if (sort)
                req.sort(sort);
            return _this.populateQuery(req, _this.populate)
                .exec(function (err, models) { return _this.handleStandardResponse(resolve, reject, err, models); });
        });
    };
    MongoDal.prototype.count = function (query) {
        var _this = this;
        return this.promise(function (resolve, reject) {
            return _this.dataModel.model.countDocuments(query)
                .exec(function (err, count) { return _this.handleStandardResponse(resolve, reject, err, count); });
        });
    };
    /**
     * Joins data of a queried row
     * @param query The query to perform
     * @param populate The populating configurations used to join other tables
     * @returns {any} The query
     */
    MongoDal.prototype.populateQuery = function (query, populate) {
        if (populate instanceof Array) {
            for (var i = 0; i < populate.length; i++) {
                query.populate(populate[i]);
            }
        }
        else if (populate) {
            query.populate(populate);
        }
        return query;
    };
    /**
     * Handles the error response sent back from a mongoose query
     * @param resolve promise resolve
     * @param reject promise reject
     * @param err The error
     * @param {any} model The model
     * @returns {ServiceResponse} A service response based on if there was an error or not
     */
    MongoDal.prototype.handleStandardResponse = function (resolve, reject, err, model) {
        if (model === void 0) { model = null; }
        if (err) {
            return reject(new server_core_web_1.ServiceResponse(err));
        }
        return resolve(new server_core_web_1.ServiceResponse(model));
    };
    /**
     * Helper function to standardize promise and it's return type
     *
     * @param func The function to call in the promise
     * @returns {Promise<ServiceResponse>} The promise
     */
    MongoDal.prototype.promise = function (func) {
        return new Promise(function (resolve, reject) { return func(resolve, reject); });
    };
    MongoDal.prototype.handleCaught = function (e, reject, res) {
        if (e instanceof server_core_web_1.ServiceResponse) {
            return reject(res);
        }
        return reject(e);
    };
    return MongoDal;
}());
exports.default = MongoDal;
